@@ -308,7 +308,12 @@ def _is_eligible(backend):
 
 
 def choose_backend(eligible_backends):
-    return eligible_backends[0]['backend']
+    """Choose the most eligible backend.
+
+    If ``eligible_backends`` is empty, then raises an IndexError.
+    """
+    bs = sorted(eligible_backends, key=itemgetter('score'), reverse=True)
+    return bs[0]['backend']
 
 
 def query_backends(path, backends, allowed_backend_names=None):
@@ -321,6 +326,10 @@ def query_backends(path, backends, allowed_backend_names=None):
             lambda b: b.name in allowed_backend_names, backends)
     results = _query_backends(path, allowed)
     ineligible, eligible = dichotomy(_is_eligible, results)
+    if not eligible:
+        raise NoEligibleBackend(
+            path, ineligible, disallowed_backends=disallowed)
+    # XXX: Can maybe get rid of this.
     eligible.sort(key=itemgetter('score'), reverse=True)
     return disallowed, ineligible, eligible
 
@@ -350,9 +359,6 @@ class BackendSelector(object):
         """
         disallowed, ineligible, eligible = query_backends(
             path, self.backends, self.allowed_backend_names)
-        if not eligible:
-            raise NoEligibleBackend(
-                path, ineligible, disallowed_backends=disallowed)
         return eligible
 
     def get_info(self, path):
