@@ -11,6 +11,7 @@ from treeshape import (
 from ..backend import (
     Backend,
     BackendSelector,
+    choose_backend,
     EXTERNAL_BACKEND_PATHS,
     ExternalHelpersBackend,
     ExternalHelpersBackendLoader,
@@ -19,6 +20,7 @@ from ..backend import (
     get_info_for,
     NoBackend,
     NoEligibleBackend,
+    query_backends,
     StaticBackend,
     StaticLoader,
     WantError,
@@ -489,6 +491,60 @@ class GetDefaultLoaderTests(TestCase):
         self.assertEqual(EXTERNAL_BACKEND_PATHS, loader.paths)
 
 
+class ChooseBackendTests(TestCase):
+
+    def test_first_backend(self):
+        b1 = object()
+        b2 = object()
+        backends = [{'score': 10, 'backend': b1}, {'score': 20, 'backend': b2}]
+        self.assertIs(b2, choose_backend(backends))
+
+    def test_no_backends(self):
+        self.assertRaises(IndexError, choose_backend, [])
+
+
+class QueryBackendsTests(TestCase):
+
+    def make_backend(self, name=None, score=0, info=None, reason=None):
+        if name is None:
+            name = self.getUniqueString('name')
+        if info is None:
+            info = {}
+        if reason is None:
+            reason = self.getUniqueString('reason')
+        return StaticBackend(name, score, info, reason=reason)
+
+    def to_query(self, backends):
+        return sorted(
+            [{'score': b.score, 'reason': b.reason, 'backend': b}
+             for b in backends])
+
+    def test_no_backends(self):
+        self.assertRaises(NoBackend, query_backends, '.', [])
+
+    def test_no_eligible_backends(self):
+        backends = [self.make_backend(score=0), self.make_backend(score=0)]
+        self.assertRaises(NoEligibleBackend, query_backends, '.', backends)
+
+    def test_eligible_backends_only(self):
+        eligible = [self.make_backend(score=10), self.make_backend(score=20)]
+        expected_eligible = self.to_query(eligible)
+        result = query_backends('.', eligible)
+        self.assertEqual(
+            ([], [], expected_eligible),
+            (result[0], result[1], sorted(result[2])))
+
+    def test_ineligible_backends(self):
+        eligible = [self.make_backend(score=10), self.make_backend(score=20)]
+        expected_eligible = self.to_query(eligible)
+        ineligible = [self.make_backend() for i in range(3)]
+        expected_ineligible = self.to_query(ineligible)
+        result = query_backends('.', eligible + ineligible)
+        self.assertEqual(
+            ([], expected_ineligible, expected_eligible),
+            (result[0], sorted(result[1]), sorted(result[2])))
+
+
 class TestSelector(BackendSelector):
 
     def get_info(self, path):
