@@ -646,3 +646,24 @@ class PackageFileGroupTests(TestCase):
         self.assertEqual('foo.txt', foo.path)
         self.assertEqual(foo_content, foo.get_contents())
         self.assertEqual(True, foo.overwrite)
+
+    def test_get_info_once(self):
+        foo_content = self.getUniqueString()
+        tree = self.useFixture(
+            FileTree(
+                {'foo': {'content': foo_content},
+                 'bar': {}}))
+        group = PackageFileGroup()
+        project_info = DictInfo({
+            ExtraFilesFromPaths.name: {
+                'foo.txt': tree.join('foo'),
+                'debian/install': tree.join('bar'),
+                }})
+        calls = []
+        orig_get_all = project_info.get_all
+        def get_all(keys):
+            calls.append(keys)
+            return orig_get_all(keys)
+        project_info.get_all = get_all
+        group.get_files(project_info)
+        self.assertEqual(1, len(calls), calls)
